

class Hook:
    def start(self, states, global_state, window):
        pass

    def tick(self, states, global_state, window):
        pass
    
    def end(self, states, global_state, window):
        pass